@@ -1,6 +1,5 @@
 #include "later.h"
 #include <Rcpp.h>
-#include <map>
 #include <queue>
 #include <memory>
 #include "debug.h"
@@ -16,7 +15,8 @@ using std::shared_ptr;
 
 static size_t exec_callbacks_reentrancy_count = 0;
 
-static CallbackRegistryTable callbackRegistryTable;
+// instance has global scope as declared in callback_registry_table.h
+CallbackRegistryTable callbackRegistryTable;
 
 
 class ProtectCallbacks {
@@ -85,13 +85,13 @@ bool at_top_level() {
 
 static int current_registry;
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 void setCurrentRegistryId(int id) {
   ASSERT_MAIN_THREAD()
   current_registry = id;
 }
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 int getCurrentRegistryId() {
   ASSERT_MAIN_THREAD()
   return current_registry;
@@ -130,14 +130,14 @@ shared_ptr<CallbackRegistry> getGlobalRegistry() {
 // parent. Any children of this registry are orphaned -- they no longer have a
 // parent. (Maybe this should be an option?)
 //
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 bool deleteCallbackRegistry(int loop_id) {
   ASSERT_MAIN_THREAD()
   if (loop_id == GLOBAL_LOOP) {
-    Rf_error("Can't delete global loop.");
+    Rf_error("Can't destroy global loop.");
   }
   if (loop_id == getCurrentRegistryId()) {
-    Rf_error("Can't delete current loop.");
+    Rf_error("Can't destroy current loop.");
   }
 
   return callbackRegistryTable.remove(loop_id);
@@ -145,33 +145,33 @@ bool deleteCallbackRegistry(int loop_id) {
 
 
 // This is called when the R loop handle is GC'd.
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 bool notifyRRefDeleted(int loop_id) {
   ASSERT_MAIN_THREAD()
   if (loop_id == GLOBAL_LOOP) {
-    Rf_error("Can't delete global loop.");
+    Rf_error("Can't notify that reference to global loop is deleted.");
   }
   if (loop_id == getCurrentRegistryId()) {
-    Rf_error("Can't delete current loop.");
+    Rf_error("Can't notify that reference to current loop is deleted.");
   }
 
   return callbackRegistryTable.notifyRRefDeleted(loop_id);
 }
 
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 void createCallbackRegistry(int id, int parent_id) {
   ASSERT_MAIN_THREAD()
   callbackRegistryTable.create(id, parent_id);
 }
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 bool existsCallbackRegistry(int id) {
   ASSERT_MAIN_THREAD()
   return callbackRegistryTable.exists(id);
 }
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 Rcpp::List list_queue_(int id) {
   ASSERT_MAIN_THREAD()
   shared_ptr<CallbackRegistry> registry = callbackRegistryTable.getRegistry(id);
@@ -200,12 +200,19 @@ bool execCallbacksOne(
   do {
     // We only take one at a time, because we don't want to lose callbacks if
     // one of the callbacks throws an error
-    std::vector<Callback_sp> callbacks = callback_registry->take(1, now);
-    if (callbacks.size() == 0) {
+    Callback_sp callback = callback_registry->pop(now);
+    if (callback == nullptr) {
       break;
     }
+
+#ifdef RCPP_USING_UNWIND_PROTECT // See https://github.com/r-lib/later/issues/191
     // This line may throw errors!
-    callbacks[0]->invoke_wrapped();
+    callback->invoke();
+#else
+    // This line may throw errors!
+    callback->invoke_wrapped();
+#endif
+
   } while (runAll);
 
   // I think there's no need to lock this since it's only modified from the
@@ -222,7 +229,7 @@ bool execCallbacksOne(
 }
 
 // Execute callbacks for an event loop and its children.
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 bool execCallbacks(double timeoutSecs, bool runAll, int loop_id) {
   ASSERT_MAIN_THREAD()
   shared_ptr<CallbackRegistry> registry = callbackRegistryTable.getRegistry(loop_id);
@@ -267,7 +274,7 @@ bool execCallbacksForTopLevel() {
   return any;
 }
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 bool idle(int loop_id) {
   ASSERT_MAIN_THREAD()
   shared_ptr<CallbackRegistry> registry = callbackRegistryTable.getRegistry(loop_id);
@@ -279,7 +286,7 @@ bool idle(int loop_id) {
 
 
 static bool initialized = false;
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 void ensureInitialized() {
   if (initialized) {
     return;
@@ -296,7 +303,7 @@ void ensureInitialized() {
   initialized = true;
 }
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 std::string execLater(Rcpp::Function callback, double delaySecs, int loop_id) {
   ASSERT_MAIN_THREAD()
   ensureInitialized();
@@ -322,7 +329,7 @@ bool cancel(uint64_t callback_id, int loop_id) {
   return registry->cancel(callback_id);
 }
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 bool cancel(std::string callback_id_s, int loop_id) {
   ASSERT_MAIN_THREAD()
   uint64_t callback_id;
@@ -340,7 +347,7 @@ bool cancel(std::string callback_id_s, int loop_id) {
 
 
 
-// [[Rcpp::export]]
+// [[Rcpp::export(rng = false)]]
 double nextOpSecs(int loop_id) {
   ASSERT_MAIN_THREAD()
   shared_ptr<CallbackRegistry> registry = callbackRegistryTable.getRegistry(loop_id);
@@ -357,12 +364,6 @@ double nextOpSecs(int loop_id) {
   }
 }
 
-// Schedules a C function to execute on the global loop. Returns callback ID
-// on success, or 0 on error.
-extern "C" uint64_t execLaterNative(void (*func)(void*), void* data, double delaySecs) {
-  return execLaterNative2(func, data, delaySecs, GLOBAL_LOOP);
-}
-
 // Schedules a C function to execute on a specific event loop. Returns
 // callback ID on success, or 0 on error.
 extern "C" uint64_t execLaterNative2(void (*func)(void*), void* data, double delaySecs, int loop_id) {
