@@ -21,29 +21,29 @@ extern void* R_GlobalContext;
 extern void* R_TopLevelContext;
 
 // Whether we have initialized the input handler.
-int initialized = 0;
+static int initialized = 0;
 
 // The handles to the read and write ends of a pipe. We use this pipe
 // to signal R's input handler callback mechanism that we want to be
 // called back.
-int pipe_in  = -1;
-int pipe_out = -1;
+static int pipe_in  = -1;
+static int pipe_out = -1;
 
-int dummy_pipe_in  = -1;
-int dummy_pipe_out = -1;
+static int dummy_pipe_in  = -1;
+static int dummy_pipe_out = -1;
 
 // Whether the file descriptor is ready for reading, i.e., whether
 // the input handler callback is scheduled to be called. We use this
 // to avoid unnecessarily writing to the pipe.
-bool hot = false;
+static bool hot = false;
 // This mutex protects reading/writing of `hot` and of reading from/writing to
 // the pipe.
-Mutex m(tct_mtx_plain);
+static Mutex m(tct_mtx_plain);
 
 // The buffer we're using for the pipe. This doesn't have to be large,
 // in theory it only ever holds zero or one byte.
-size_t BUF_SIZE = 256;
-void *buf;
+static size_t BUF_SIZE = 256;
+static void *buf;
 
 void set_fd(bool ready) {
   Guard g(&m);
@@ -55,9 +55,7 @@ void set_fd(bool ready) {
       hot = true;
     } else {
       if (read(pipe_out, buf, BUF_SIZE) < 0) {
-        // TODO: This sets a warning but it doesn't get displayed until
-        // after the next R command is executed. Can we make it sooner?
-        Rf_warning("Failed to read out of pipe for later package");
+        Rf_warningcall_immediate(R_NilValue, "Failed to read out of pipe for later package");
       }
       hot = false;
     }
@@ -101,7 +99,7 @@ static void async_input_handler(void *data) {
     // Instead, we set the file descriptor to cold, and tell the timer to fire
     // again in a few milliseconds. This should give enough breathing room that
     // we don't interfere with the sockets too much.
-    timer.set(Timestamp(0.032));
+    timer.set(Timestamp(0.01));
     return;
   }
 
@@ -128,20 +126,24 @@ static void async_input_handler(void *data) {
     DEBUG_LOG("async_input_handler: caught Rcpp::internal::InterruptedException", LOG_INFO);
     REprintf("later: interrupt occurred while executing callback.\n");
   }
+  catch(Rcpp::LongjumpException& e){
+    DEBUG_LOG("async_input_handler: caught exception", LOG_INFO);
+    REprintf("later: exception occurred while executing callback.\n");
+  }
   catch(std::exception& e){
     DEBUG_LOG("async_input_handler: caught exception", LOG_INFO);
     std::string msg = "later: exception occurred while executing callback: \n";
     msg += e.what();
     msg += "\n";
-    REprintf(msg.c_str());
+    REprintf("%s", msg.c_str());
   }
   catch( ... ){
     REprintf("later: c++ exception (unknown reason) occurred while executing callback.\n");
   }
 }
 
-InputHandler* inputHandlerHandle;
-InputHandler* dummyInputHandlerHandle;
+static InputHandler* inputHandlerHandle;
+static InputHandler* dummyInputHandlerHandle;
 
 // If the real input handler has been removed, the dummy input handler removes
 // itself. The real input handler cannot remove both; otherwise a segfault
@@ -173,6 +175,8 @@ void child_proc_after_fork() {
       close(pipe_out);
       pipe_out = -1;
     }
+
+    removeInputHandler(&R_InputHandlers, dummyInputHandlerHandle);
     if (dummy_pipe_in  > 0) {
       close(dummy_pipe_in);
       dummy_pipe_in  = -1;
@@ -243,6 +247,7 @@ void deInitialize() {
     // Trigger remove_dummy_handler()
     // Store `ret` because otherwise it raises a significant warning.
     ssize_t ret = write(dummy_pipe_in, "a", 1);
+    (void)ret; // squelch compiler warning
   }
 }
 
